import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;

class RequestClient {
  factory RequestClient() {
    return _instance;
  }

  RequestClient._internal();

  static final RequestClient _instance = RequestClient._internal();
  http.Client _client = http.Client();

  static const String userAgent = 'User-Agent';
  static const String contentType = 'Content-Type';
  static const String accept = 'Accept';

  Future<Map<String, String>> _createCommonHeader() async {
    final common = <String, String>{};
    common[accept] = ContentType.json.toString();
    return common;
  }

  Future<Map<String, String>> _createHeader() async {
    final common = <String, String>{};
    common[contentType] = ContentType.json.toString();
    return common;
  }

  Future<http.Response> get(String url, Map<String, String> headers) async {
    debugPrint('-------------------------------');
    debugPrint('GET Request');
    debugPrint(url);
    debugPrint(headers.toString());
    debugPrint('-------------------------------');
    headers.addAll(await _createCommonHeader());
    final response = await _client.get(Uri.parse(url), headers: headers);
    return response;
  }

  Future<http.Response> post(
      String url, Map<String, String> headers, String body) async {
    debugPrint('-------------------------------');
    debugPrint('POST Request');
    debugPrint(url);
    debugPrint(headers.toString());
    debugPrint('-------------------------------');
    headers.addAll(await _createCommonHeader());
    final response = await _client.post(
      Uri.parse(url),
      headers: headers,
      body: body,
    );
    return response;
  }
}
