import 'dart:convert';

import 'package:event_analysis_library/src/api/request.dart';
import 'package:event_analysis_library/src/api/request_client.dart';
import 'package:event_analysis_library/src/api/response.dart';
import 'package:event_analysis_library/src/shared/api_constants.dart';
import 'package:event_analysis_library/src/shared/model/sdk_initialization/sdk_initialization.dart';
import 'package:event_analysis_library/src/shared/utils.dart';

class SdkInitializationRequest implements Request {
  final String productKey;

  SdkInitializationRequest(this.productKey);

  @override
  String getBody() {
    final body = getJson();
    return body;
  }

  String getJson() {
    return jsonEncode({});
  }

  @override
  Map<String, String> getHeaders() {
    String authentication = Utils().createCredential(ApiConstants.USER_NAME, ApiConstants.USER_PASSWORD);
    final headers = <String, String>{};
    headers['Content-Type'] = 'application/json';
    headers['Authorization'] = authentication;
    return headers;
  }

  @override
  String getUrl() {
    return Request.createUrl('sdk_info/$productKey');
  }

  Future<SdkInitializationResponse> execute() async {
    final result = SdkInitializationResponse();

    await RequestClient().get(getUrl(), getHeaders()).then((response) {
      if(response.statusCode == 200){
        final dynamic object = json.decode(response.body);
        final code = SdkInitialization.fromJson(object as Map<String, dynamic>);
        if(code.statusCode == 200)
          result
            ..hasError = false
            ..errorMessage = ""
            ..response = code;
        else
          result
            ..hasError = true
            ..errorMessage = code.msg;
      }else{
        result
          ..hasError = true
          ..errorMessage = "${response.statusCode} ERROR FROM API";
      }
    }).catchError((Object error){
      result
        ..hasError = true
        ..errorMessage = Response.getErrorMessage(error);
    });

    return result;
  }
}

class SdkInitializationResponse extends Response {
  SdkInitializationResponse({this.hasError = false, this.errorMessage = '', this.response});

  bool hasError;
  String? errorMessage;
  SdkInitialization? response;
}
