import 'dart:convert';

import 'package:event_analysis_library/src/api/request.dart';
import 'package:event_analysis_library/src/api/request_client.dart';
import 'package:event_analysis_library/src/api/response.dart';
import 'package:event_analysis_library/src/shared/api_constants.dart';
import 'package:event_analysis_library/src/shared/utils.dart';

class UserIdentityRequest implements Request {
  final String productKey;
  final String token;
  final String userIdentity;

  UserIdentityRequest({required this.productKey, required this.token, required this.userIdentity});

  @override
  String getBody() {
    final body = getJson();
    return body;
  }

  String getJson() {
    return jsonEncode({});
  }

  @override
  Map<String, String> getHeaders() {
    String authentication = Utils().createCredential(ApiConstants.USER_NAME, ApiConstants.USER_PASSWORD);
    return Request.createAuthHeader(authentication, token);
  }

  @override
  String getUrl() {
    return Request.createUrl('identity/$productKey');
  }

  execute() async {
    final result = UserIdentityResponse();

    await RequestClient().post(getUrl(), getHeaders(), userIdentity).then((response) {
      if(response.statusCode == 200){
        final dynamic object = json.decode(response.body);
        if(object['statusCode'] == 200){
          result
            ..hasError = false
            ..errorMessage = ""
            ..response = object;
        }else{
          result
            ..hasError = false
            ..errorMessage = object['msg'];
        }
      }else if (response.statusCode == 504){
        result
          ..hasError = true
          ..errorMessage = "${response.statusCode} Connection Timeout";
      }else {
        result
          ..hasError = true
          ..errorMessage = "${response.statusCode} Something went wrong";
      }
    }).catchError((Object error){
      result
        ..hasError = true
        ..errorMessage = Response.getErrorMessage(error);
    });

    return result;
  }
}

class UserIdentityResponse extends Response {
  UserIdentityResponse({this.hasError = false, this.errorMessage = '', this.response});

  bool hasError;
  String? errorMessage;
  dynamic response;
}
