import 'dart:convert';
import 'dart:io';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:event_analysis_library/src/shared/model/event/event_model.dart';
import 'package:event_analysis_library/src/shared/model/meta/device_info_model.dart';
import 'package:event_analysis_library/src/shared/model/meta/device_meta_model.dart';
import 'package:flutter/cupertino.dart';
import 'package:geolocator/geolocator.dart';
import 'package:package_info_plus/package_info_plus.dart';

class Utils {

  getDeviceMeta(String sesId, Position? location, String locale) async {
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    var connectivityResult = await (Connectivity().checkConnectivity());
    var mediaQuery = MediaQueryData.fromWindow(WidgetsBinding.instance!.window);

    DeviceMeta deviceMeta = DeviceMeta(
      os: Platform.isAndroid ? "Android" : "IOS",
      osVersion: Platform.operatingSystemVersion,
      applicationid: "",
      versionNum: packageInfo.version,
      versionCode: packageInfo.buildNumber,
      flavor: "",
      width: mediaQuery.size.width.toString(),
      height: mediaQuery.size.height.toString(),
      density: ((mediaQuery.size.width * mediaQuery.devicePixelRatio).toStringAsFixed(0) + "x" + (mediaQuery.size.height * mediaQuery.devicePixelRatio).toStringAsFixed(0)),
      densityDpi: (mediaQuery.devicePixelRatio * 160).toString(),
      orientation: mediaQuery.orientation == Orientation.portrait
          ? "Portrait"
          : "Landscape",
      locale: locale,
      latitude: location == null ? "" : location.latitude.toString(),
      longitude: location == null ? "" : location.longitude.toString(),
      batteryLevel: "",
      batteryTemp: "",
      network: connectivityResult == ConnectivityResult.mobile
          ? "Data"
          : connectivityResult == ConnectivityResult.wifi
              ? "WIFI"
              : "n/a",
      connectedToNetwork: "",
      deviceId: "",
      carrier: "",
      ram: "",
      internalMemTotal: "",
      internalMemAvail: "",
      sessionId: sesId,
      brand: "",
      device: "",
      deviceType: "",
    );

    return json.encode(DeviceInfo(deviceMeta).toJson());
  }

  getEventData(String eventName, String eventValue, String userId, String sessionId, Offset position){
    Event event = Event(eventName, eventValue, userId, sessionId, position.dx.toString(), position.dy.toString());
    return json.encode(event.toJson());
  }

  getSessionId(){
    return DateTime.now().millisecondsSinceEpoch.toString();
  }

  isLocationEnabled() async {
    bool enabled = await Geolocator.isLocationServiceEnabled();
    return enabled;
  }

  createCredential(String username, String password){
    Codec<String, String> stringToBase64 = latin1.fuse(base64);
    String credentials = "$username:$password";
    String encoded = stringToBase64.encode(credentials);
    return "Basic $encoded";
  }
}

